@@ -4,7 +4,7 @@ type mkdir_result =
   | Already_exists (** The directory already exists. No action was taken. *)
   | Created (** The directory was created. *)
   | Missing_parent_directory
-      (** No parent directory, use [mkdir_p] if you want to create it too. *)
+  (** No parent directory, use [mkdir_p] if you want to create it too. *)
 
 val mkdir : ?perms:int -> string -> mkdir_result
 
@@ -27,8 +27,18 @@ val follow_symlink : string -> (string, follow_symlink_error) result
     [Error Max_depth_exceeded] on some intermediate path). *)
 val follow_symlinks : string -> string option
 
-val unlink : string -> unit
+val unlink_exn : string -> unit
 val unlink_no_err : string -> unit
+
+type unlink_status =
+  | Success
+  | Does_not_exist
+  | Is_a_directory
+  | Error of exn
+
+(** Unlink and return error, if any. *)
+val unlink : string -> unlink_status
+
 val initial_cwd : string
 
 type clear_dir_result =
@@ -41,3 +51,9 @@ val clear_dir : string -> clear_dir_result
 val rm_rf : string -> unit
 
 val is_root : string -> bool
+
+val traverse_files
+  :  dir:string
+  -> init:'acc
+  -> f:(dir:string -> Filename.t -> 'acc -> 'acc)
+  -> 'acc
