@@ -354,8 +354,8 @@ end = struct
   include (
     Local_gen :
       module type of Local_gen
-        with type 'a t := 'a Local_gen.t
-        with module Prefix := Local_gen.Prefix)
+      with type 'a t := 'a Local_gen.t
+      with module Prefix := Local_gen.Prefix)
 
   type nonrec t = w Local_gen.t
 
@@ -401,9 +401,13 @@ end = struct
   type t = string
 
   let drop_prefix path ~prefix =
-    String.drop_prefix path ~prefix
-    |> Option.map ~f:(fun p ->
-      String.drop_prefix_if_exists ~prefix:"/" p |> Local.of_string)
+    if prefix = path
+    then Some Local.root
+    else (
+      let prefix = prefix ^ "/" in
+      let open Option.O in
+      let+ suffix = String.drop_prefix path ~prefix in
+      Local.of_string suffix)
   ;;
 
   let to_string t = t
@@ -509,6 +513,7 @@ module Source0 = struct
   include Local
 
   let to_dyn s = Dyn.variant "In_source_tree" [ to_dyn s ]
+  let append_local = Local.append
 end
 
 let abs_root, set_root =
@@ -729,6 +734,7 @@ module Build = struct
   let of_local t = t
   let chmod t ~mode = Unix.chmod (to_string t) mode
   let lstat t = Unix.lstat (to_string t)
+  let unlink t = Fpath.unlink (to_string t)
   let unlink_no_err t = Fpath.unlink_no_err (to_string t)
   let to_dyn s = Dyn.variant "In_build_dir" [ to_dyn s ]
 end
@@ -1129,8 +1135,16 @@ let is_directory t =
 ;;
 
 let rmdir t = Unix.rmdir (to_string t)
-let unlink t = Fpath.unlink (to_string t)
-let link x y = Unix.link (to_string x) (to_string y)
+let unlink_exn t = Fpath.unlink_exn (to_string t)
+
+let link src dst =
+  match Unix.link (to_string src) (to_string dst) with
+  | exception Unix.Unix_error (Unix.EUNKNOWNERR -1142, syscall, arg)
+  (* Needed for OCaml < 5.1 on windows *) ->
+    Exn.reraise (Unix.Unix_error (Unix.EMLINK, syscall, arg))
+  | s -> s
+;;
+
 let unlink_no_err t = Fpath.unlink_no_err (to_string t)
 let build_dir_exists () = is_directory build_dir
 
@@ -1176,28 +1190,6 @@ let mkdir_p ?perms = function
       (Outside_build_dir.append_local (Fdecl.get Build.build_dir) k)
 ;;
 
-let touch ?(create = true) p =
-  let p =
-    match p with
-    | External s -> External.to_string s
-    | In_source_tree s -> Local_gen.to_string s
-    | In_build_dir k ->
-      Outside_build_dir.to_string
-        (Outside_build_dir.append_local (Fdecl.get Build.build_dir) k)
-  in
-  let create =
-    if create
-    then fun () -> Unix.close (Unix.openfile p [ Unix.O_CREAT ] 0o777)
-    else Fun.id
-  in
-  try Unix.utimes p 0.0 0.0 with
-  | Unix.Unix_error (Unix.ENOENT, _, _) -> create ()
-  | Unix.Unix_error (Unix.EUNKNOWNERR 0, _, _) when Sys.win32 && not (Sys.file_exists p)
-    ->
-    (* OCaml PR#8857 *)
-    create ()
-;;
-
 let extension t =
   match t with
   | External t -> External.extension t
@@ -1356,14 +1348,18 @@ let set_of_build_paths_list =
 ;;
 
 let set_of_external_paths set = External.Set.to_list set |> Set.of_list_map ~f:external_
-let rename old_path new_path = Sys.rename (to_string old_path) (to_string new_path)
+let rename old_path new_path = Unix.rename (to_string old_path) (to_string new_path)
 let chmod t ~mode = Unix.chmod (to_string t) mode
 let follow_symlink path = Fpath.follow_symlink (to_string path) |> Result.map ~f:of_string
 
 let drop_prefix path ~prefix =
-  String.drop_prefix (to_string path) ~prefix:(to_string prefix)
-  |> Option.map ~f:(fun p ->
-    String.drop_prefix_if_exists ~prefix:"/" p |> Local.of_string)
+  if prefix = path
+  then Some Local.root
+  else (
+    let prefix = to_string prefix ^ "/" in
+    let open Option.O in
+    let+ suffix = String.drop_prefix (to_string path) ~prefix in
+    Local.of_string suffix)
 ;;
 
 let drop_prefix_exn t ~prefix =
